@@ -73,8 +73,8 @@ static void         gst_play_init		    (GstPlay *play);
 static void         gst_play_class_init	            (GstPlayClass *klass);
 static void         gst_play_dispose		    (GObject *object);
 
-static void         gst_play_default_timeout_add    (guint interval, GSourceFunc function, gpointer data);
-static void         gst_play_default_idle_add       (GSourceFunc function, gpointer data);
+static guint         gst_play_default_timeout_add    (guint interval, GSourceFunc function, gpointer data);
+static guint         gst_play_default_idle_add       (GSourceFunc function, gpointer data);
 
 static void         gst_play_set_property	    (GObject *object, guint prop_id, 
 						     const GValue *value, GParamSpec *pspec);
@@ -297,7 +297,7 @@ gst_play_init (GstPlay *play)
 	play->audio_bin_mutex = g_mutex_new();
 	play->video_bin_mutex = g_mutex_new();
 	gst_play_set_idle_timeout_funcs(play, gst_play_default_timeout_add, gst_play_default_idle_add);
-
+	play->tick_timeout_id = play->idle_timeout_id = play->idle_signal_id = 0;
 }
 
 GstPlay *
@@ -312,21 +312,16 @@ gst_play_new (GstPlayPipeType pipe_type, GError **error)
 		case GST_PLAY_PIPE_VIDEO:
 			play->setup_pipeline = gst_play_video_setup;
 			play->teardown_pipeline = NULL;
+			play->set_data_src = gst_play_video_set_data_src;
 			play->set_autoplugger = gst_play_video_set_auto;
 			play->set_video_sink = gst_play_video_set_video;
 			play->set_audio_sink = gst_play_video_set_audio;
 			break;
-		case GST_PLAY_PIPE_VIDEO_THREADSAFE:
-			play->setup_pipeline = gst_play_videots_setup;
-			play->teardown_pipeline = NULL;
-			play->set_autoplugger = gst_play_videots_set_auto;
-			play->set_video_sink = gst_play_videots_set_video;
-			play->set_audio_sink = gst_play_videots_set_audio;
-			break;
 		case GST_PLAY_PIPE_AUDIO:
 			/* we can reuse the threaded set functions */
 			play->setup_pipeline = gst_play_audio_setup;
 			play->teardown_pipeline = NULL;
+			play->set_data_src = gst_play_simple_set_data_src;
 			play->set_autoplugger = gst_play_audiot_set_auto;
 			play->set_video_sink = NULL;
 			play->set_audio_sink = gst_play_audiot_set_audio;
@@ -334,6 +329,7 @@ gst_play_new (GstPlayPipeType pipe_type, GError **error)
 		case GST_PLAY_PIPE_AUDIO_THREADED:
 			play->setup_pipeline = gst_play_audiot_setup;
 			play->teardown_pipeline = NULL;
+			play->set_data_src = gst_play_simple_set_data_src;
 			play->set_autoplugger = gst_play_audiot_set_auto;
 			play->set_video_sink = NULL;
 			play->set_audio_sink = gst_play_audiot_set_audio;
@@ -341,6 +337,7 @@ gst_play_new (GstPlayPipeType pipe_type, GError **error)
 		case GST_PLAY_PIPE_AUDIO_HYPER_THREADED:
 			play->setup_pipeline = gst_play_audioht_setup;
 			play->teardown_pipeline = NULL;
+			play->set_data_src = gst_play_simple_set_data_src;
 			play->set_autoplugger = gst_play_audioht_set_auto;
 			play->set_video_sink = NULL;
 			play->set_audio_sink = gst_play_audioht_set_audio;
@@ -390,6 +387,18 @@ static void
 gst_play_dispose (GObject *object)
 {
 	GstPlay *play = GST_PLAY (object);
+	/* Before disposing, removing any callback that could
+	be called : time ticks, signal poller, etc...*/
+	if (play->tick_timeout_id)
+		if (!g_source_remove(play->tick_timeout_id))
+			g_warning("failed to remove timetick timer %d", play->tick_timeout_id);
+	if (play->idle_timeout_id)
+		if (!g_source_remove(play->idle_timeout_id))
+			g_warning("failed to remove idle timer %d", play->idle_timeout_id);
+	if (play->idle_signal_id)
+		if (!g_source_remove(play->idle_signal_id))
+			g_warning("failed to remove signal idle timer %d", play->idle_signal_id);
+		
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 	g_mutex_free(play->audio_bin_mutex);
 	g_mutex_free(play->video_bin_mutex);
@@ -413,7 +422,10 @@ callback_pipeline_deep_notify (GstElement *element, GstElement *orig, GParamSpec
 	signal->signal_data.info.element = orig;
 	signal->signal_data.info.param = param;
 	g_async_queue_push(play->signal_queue, signal);
-	play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
+	/* If an idle callback has been added for signal polling
+	no need to put a new one */
+	if (!play->idle_signal_id)
+		play->idle_signal_id = play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
 }
 
 static void
@@ -431,12 +443,12 @@ callback_pipeline_state_change (GstElement *element, GstElementState old, GstEle
 	if (GST_IS_PIPELINE (play->pipeline)){
 		switch (state) {
 		case GST_STATE_PLAYING:
-			play->idle_add_func ((GSourceFunc) gst_play_idle_callback, play);
-			play->timeout_add_func (200, (GSourceFunc) gst_play_tick_callback, play);
+			play->idle_timeout_id = play->idle_add_func ((GSourceFunc) gst_play_idle_callback, play);
+			play->tick_timeout_id = play->timeout_add_func (200, (GSourceFunc) gst_play_tick_callback, play);
 			if (play->length_nanos == 0LL){
 				/* try to get the length up to 16 times */
 				play->get_length_attempt = 16;
-				play->timeout_add_func (200, (GSourceFunc) gst_play_get_length_callback, play);
+				play->timeout_add_func (200, (GSourceFunc) gst_play_get_length_callback, play); 
 			}
 			break;
 		default:
@@ -448,7 +460,10 @@ callback_pipeline_state_change (GstElement *element, GstElementState old, GstEle
 	signal->signal_data.state.old_state = old;
 	signal->signal_data.state.new_state = state;
 	g_async_queue_push(play->signal_queue, signal);
-	play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
+	/* If an idle callback has been added for signal polling
+	no need to put a new one */
+	if (!play->idle_signal_id)
+		play->idle_signal_id = play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
 }
 
 static gboolean
@@ -456,7 +471,9 @@ gst_play_idle_signal (GstPlay *play)
 {
 	GstPlaySignal *signal;
 	gint queue_length;
-
+	
+	g_return_val_if_fail(GST_IS_PLAY(play), FALSE);
+	
 	signal = g_async_queue_try_pop(play->signal_queue);
 	if (signal == NULL){
 		return FALSE;
@@ -485,6 +502,13 @@ gst_play_idle_signal (GstPlay *play)
 
 	g_free(signal);
 	queue_length = g_async_queue_length (play->signal_queue);
+	
+	/* If queue length is zero the idle callback will be
+	destroyed */
+	
+	if (!queue_length)
+		play->idle_signal_id = 0;
+	
 	return (queue_length > 0);
 }
 
@@ -509,7 +533,10 @@ callback_video_have_xid (GstElement *element, gint xid, GstPlay *play)
 	signal->signal_id = HAVE_XID;
 	signal->signal_data.video_xid.xid = xid;
 	g_async_queue_push(play->signal_queue, signal);
-	play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
+	/* If an idle callback has been added for signal polling
+	no need to put a new one */
+	if (!play->idle_signal_id)
+		play->idle_signal_id = play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
 	/*g_print("have xid %d\n", xid);*/
 }
 
@@ -522,7 +549,10 @@ callback_video_have_size (GstElement *element, gint width, gint height, GstPlay
 	signal->signal_data.video_size.width = width;
 	signal->signal_data.video_size.height = height;
 	g_async_queue_push(play->signal_queue, signal);
-	play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
+	/* If an idle callback has been added for signal polling
+	no need to put a new one */
+	if (!play->idle_signal_id)
+		play->idle_signal_id = play->idle_add_func ((GSourceFunc) gst_play_idle_signal, play);
 	/*g_print("have size %d x %d\n", width, height);*/
 }
 
@@ -575,6 +605,9 @@ static gboolean
 gst_play_tick_callback (GstPlay *play)
 {
 	gint secs;
+	
+	g_return_val_if_fail(GST_IS_PLAY(play), FALSE);
+	
 	play->clock = gst_bin_get_clock (GST_BIN (play->pipeline));
 	play->time_nanos = gst_clock_get_time(play->clock);
 	secs = (gint) (play->time_nanos / GST_SECOND);
@@ -589,6 +622,8 @@ gst_play_tick_callback (GstPlay *play)
 static gboolean
 gst_play_idle_callback (GstPlay *play)
 {
+	g_return_val_if_fail(GST_IS_PLAY(play), FALSE);
+	
 	return gst_bin_iterate (GST_BIN (play->pipeline));
 }
 
@@ -702,22 +737,24 @@ gst_play_default_idle (GstPlayIdleData *idle_data)
 	return FALSE;
 }
 
-static void
+static guint
 gst_play_default_timeout_add (guint interval, GSourceFunc function, gpointer data)
 {
 	GstPlayIdleData *idle_data = g_new0(GstPlayIdleData, 1);
 	idle_data->func = function;
 	idle_data->data = data;
-	g_timeout_add (interval, (GSourceFunc)gst_play_default_idle, idle_data);
+
+	return g_timeout_add (interval, (GSourceFunc)gst_play_default_idle, idle_data);
 }
 
-static void
+static guint
 gst_play_default_idle_add (GSourceFunc function, gpointer data)
 {
 	GstPlayIdleData *idle_data = g_new0(GstPlayIdleData, 1);
 	idle_data->func = function;
 	idle_data->data = data;
-	g_idle_add ((GSourceFunc)gst_play_default_idle, idle_data);
+	
+	return g_idle_add ((GSourceFunc)gst_play_default_idle, idle_data);
 }
 
 void
@@ -767,6 +804,24 @@ gst_play_get_sink_element (GstPlay *play, GstElement *element){
 	return NULL;
 }
 
+gboolean
+gst_play_set_data_src (GstPlay *play, GstElement *data_src)
+{
+	g_return_val_if_fail (GST_IS_PLAY (play), FALSE);
+	g_return_val_if_fail (GST_IS_ELEMENT (data_src), FALSE);
+
+	if (gst_play_get_state (play) != GST_STATE_READY){
+		gst_play_set_state (play, GST_STATE_READY);
+	}
+
+	if (play->set_data_src){
+		return play->set_data_src(play, data_src);
+	}
+
+	/* if there is no set_data_src func, fail quietly */
+	return FALSE;
+}
+
 gboolean
 gst_play_set_video_sink (GstPlay *play, GstElement *video_sink)
 {
