@@ -442,9 +442,9 @@ done:
 }
 
 static void
-gst_avimux_pad_connect (GstPad   *pad,
-                        GstPad   *peer,
-                        gpointer  data)
+gst_avimux_pad_link (GstPad    *pad,
+                     GstPad    *peer,
+                     gpointer   data)
 {
   GstAviMux *avimux = GST_AVIMUX(data);
   const gchar *padname = gst_pad_get_name (pad);
@@ -530,8 +530,8 @@ gst_avimux_request_new_pad (GstElement     *element,
     return NULL;
   }
 
-  g_signal_connect(newpad, "connected",
-    G_CALLBACK(gst_avimux_pad_connect), (gpointer)avimux);
+  g_signal_connect(newpad, "linked",
+    G_CALLBACK(gst_avimux_pad_link), (gpointer)avimux);
   g_signal_connect(newpad, "unlinked",
     G_CALLBACK(gst_avimux_pad_unlink), (gpointer)avimux);
   gst_pad_set_link_function (newpad, gst_avimux_sinkconnect);
@@ -925,7 +925,7 @@ gst_avimux_start_file (GstAviMux *avimux)
   avimux->idx = NULL;
 
   /* header */
-  avimux->avi_hdr.streams = avimux->video_pad_connected?1:0 + avimux->audio_pad_connected?1:0;
+  avimux->avi_hdr.streams = (avimux->video_pad_connected?1:0) + (avimux->audio_pad_connected?1:0);
   avimux->is_bigfile = FALSE;
 
   header = gst_avimux_riff_get_avi_header(avimux);
@@ -1042,6 +1042,8 @@ gst_avimux_fill_queue (GstAviMux *avimux)
   GstBuffer *buffer;
 
   if (!avimux->audio_buffer_queue &&
+       avimux->audiosinkpad &&
+       avimux->audio_pad_connected &&
        GST_PAD_IS_USABLE(avimux->audiosinkpad) &&
       !avimux->audio_pad_eos)
   {
@@ -1061,6 +1063,8 @@ gst_avimux_fill_queue (GstAviMux *avimux)
   }
 
   if (!avimux->video_buffer_queue &&
+       avimux->videosinkpad &&
+       avimux->video_pad_connected &&
        GST_PAD_IS_USABLE(avimux->videosinkpad) &&
       !avimux->video_pad_eos)
   {
