@@ -343,9 +343,13 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse *mpeg_parse, GstBuffer *buffer)
     scr |= (scr1 & 0x00fffe00) << 6;
     scr |= (scr1 & 0x000000ff) << 7;
     scr |= (scr2 & 0xfe000000) >> 25;
-    
+
     buf += 5;
-    new_rate = (GUINT32_FROM_BE ((*(guint32 *) buf)) & 0x7ffffe00) >> 9;
+    /* we do this byte by byte because buf[3] might be outside of buf's
+     * memory space */
+    new_rate = (buf[0] & 0x7f) << 15;
+    new_rate |= buf[1] << 7;
+    new_rate |= buf[2] >> 1;
   }
 
   scr_orig = scr;
@@ -408,7 +412,15 @@ gst_mpeg_parse_loop (GstElement *element)
   gboolean mpeg2;
   GstClockTime time;
 
+  /* maybe later we can make this less fatal, but right now this shouldn't
+   * happen at all */
+  g_assert (GST_IS_ELEMENT (element));
+
   data = gst_mpeg_packetize_read (mpeg_parse->packetize);
+  if (data == NULL) {
+    g_warning ("gst_mpeg_packetize_read returned a NULL buffer, not good !");
+    return;
+  }
 
   id = GST_MPEG_PACKETIZE_ID (mpeg_parse->packetize);
   mpeg2 = GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize);
@@ -536,13 +548,13 @@ gst_mpeg_parse_loop (GstElement *element)
           /* 
 	   * The mpeg spec says something like this, but that doesn't really work:
 	   *
-	   * mpeg_parse->next_scr = (scr * br + bss * 90000LL) / (90000LL + br);
+	   * mpeg_parse->next_scr = (scr * br + bss * CLOCK_FREQ) / (CLOCK_FREQ + br);
 	   */
-          mpeg_parse->next_scr = scr + (bss * 90000LL) / br;
+          mpeg_parse->next_scr = scr + (bss * CLOCK_FREQ) / br;
         }
         else {
           /* we are interpolating the scr here */
-          mpeg_parse->next_scr = scr + (bss * 90000LL) / br;
+          mpeg_parse->next_scr = scr + (bss * CLOCK_FREQ) / br;
         }
       }
       else {
