@@ -49,13 +49,13 @@ static GstClockTime 		gst_osssink_get_time 		(GstClock *clock, gpointer data);
 static const GstFormat* 	gst_osssink_get_formats 	(GstPad *pad);
 static gboolean 		gst_osssink_convert 		(GstPad *pad, GstFormat src_format, gint64 src_value,
 	            						 GstFormat *dest_format, gint64 *dest_value);
-static const GstPadQueryType* 	gst_osssink_get_query_types 	(GstPad *pad);
-static gboolean 		gst_osssink_query 		(GstElement *element, GstPadQueryType type, 
+static const GstQueryType* 	gst_osssink_get_query_types 	(GstPad *pad);
+static gboolean 		gst_osssink_query 		(GstElement *element, GstQueryType type, 
 								 GstFormat *format, gint64 *value);
-static gboolean 		gst_osssink_sink_query 		(GstPad *pad, GstPadQueryType type,
+static gboolean 		gst_osssink_sink_query 		(GstPad *pad, GstQueryType type,
 								 GstFormat *format, gint64 *value);
 
-static GstPadConnectReturn	gst_osssink_sinkconnect		(GstPad *pad, GstCaps *caps);
+static GstPadLinkReturn	gst_osssink_sinkconnect		(GstPad *pad, GstCaps *caps);
 
 static void 			gst_osssink_set_property	(GObject *object, guint prop_id, const GValue *value, 
 		  						 GParamSpec *pspec);
@@ -207,7 +207,7 @@ gst_osssink_init (GstOssSink *osssink)
   osssink->sinkpad = gst_pad_new_from_template (
 		  GST_PAD_TEMPLATE_GET (osssink_sink_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (osssink), osssink->sinkpad);
-  gst_pad_set_connect_function (osssink->sinkpad, gst_osssink_sinkconnect);
+  gst_pad_set_link_function (osssink->sinkpad, gst_osssink_sinkconnect);
   gst_pad_set_bufferpool_function (osssink->sinkpad, gst_osssink_get_bufferpool);
   gst_pad_set_convert_function (osssink->sinkpad, gst_osssink_convert);
   gst_pad_set_query_function (osssink->sinkpad, gst_osssink_sink_query);
@@ -230,31 +230,22 @@ gst_osssink_init (GstOssSink *osssink)
 }
 
 
-static GstPadConnectReturn 
+static GstPadLinkReturn 
 gst_osssink_sinkconnect (GstPad *pad, GstCaps *caps) 
 {
   GstOssSink *osssink = GST_OSSSINK (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps))
-  {
-    GST_DEBUG (GST_CAT_NEGOTIATION, "caps not fixed, connection delayed");
-    return GST_PAD_CONNECT_DELAYED;
-  }
+    return GST_PAD_LINK_DELAYED;
 
   if (!gst_osscommon_parse_caps (&osssink->common, caps))
-  {
-    GST_DEBUG (GST_CAT_NEGOTIATION, 
-	       "caps couldn't be parsed, connection refused");
-    return GST_PAD_CONNECT_REFUSED;
-  }
+    return GST_PAD_LINK_REFUSED;
 
   if (!gst_osscommon_sync_parms (&osssink->common)) {
-    GST_DEBUG (GST_CAT_NEGOTIATION, 
-	       "caps couldn't sync parameters, connection refused");
-    return GST_PAD_CONNECT_REFUSED;
+    return GST_PAD_LINK_REFUSED;
   }
 
-  return GST_PAD_CONNECT_OK;
+  return GST_PAD_LINK_OK;
 }
 
 static inline gint64 
@@ -384,8 +375,10 @@ gst_osssink_chain (GstPad *pad, GstBuffer *buf)
 	queued = delay * GST_SECOND / osssink->common.bps;
 
 	if  (osssink->resync && osssink->sync) {
-	  gst_element_clock_wait (GST_ELEMENT (osssink), osssink->clock, 
-				buftime - queued, &jitter);
+          GstClockID id = gst_clock_new_single_shot_id (osssink->clock, buftime - queued);
+
+	  gst_element_clock_wait (GST_ELEMENT (osssink), id, &jitter);
+	  gst_clock_id_free (id);
 
 	  if (jitter >= 0) {
             gst_clock_handle_discont (osssink->clock, buftime - queued + jitter);
@@ -439,19 +432,19 @@ gst_osssink_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
 		                dest_format, dest_value);
 }
 
-static const GstPadQueryType*
+static const GstQueryType*
 gst_osssink_get_query_types (GstPad *pad)
 {
-  static const GstPadQueryType query_types[] = {
-    GST_PAD_QUERY_LATENCY,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType query_types[] = {
+    GST_QUERY_LATENCY,
+    GST_QUERY_POSITION,
     0,
   };
   return query_types;
 }
 
 static gboolean
-gst_osssink_sink_query (GstPad *pad, GstPadQueryType type, GstFormat *format, gint64 *value) 
+gst_osssink_sink_query (GstPad *pad, GstQueryType type, GstFormat *format, gint64 *value) 
 {
   gboolean res = TRUE;
   GstOssSink *osssink;
@@ -459,7 +452,7 @@ gst_osssink_sink_query (GstPad *pad, GstPadQueryType type, GstFormat *format, gi
   osssink = GST_OSSSINK (gst_pad_get_parent (pad));
   
   switch (type) {
-    case GST_PAD_QUERY_LATENCY:
+    case GST_QUERY_LATENCY:
       if (!gst_osssink_convert (pad, 
 			        GST_FORMAT_BYTES, gst_osssink_get_delay (osssink),
 		                format, value)) 
@@ -467,7 +460,7 @@ gst_osssink_sink_query (GstPad *pad, GstPadQueryType type, GstFormat *format, gi
         res = FALSE;
       }
       break;
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       if (!gst_osssink_convert (pad, 
 			        GST_FORMAT_TIME, gst_clock_get_time (osssink->provided_clock),
 		                format, value)) 
@@ -484,7 +477,7 @@ gst_osssink_sink_query (GstPad *pad, GstPadQueryType type, GstFormat *format, gi
 }
 
 static gboolean
-gst_osssink_query (GstElement *element, GstPadQueryType type, GstFormat *format, gint64 *value) 
+gst_osssink_query (GstElement *element, GstQueryType type, GstFormat *format, gint64 *value) 
 {
   GstOssSink *osssink = GST_OSSSINK (element);
 
@@ -598,12 +591,7 @@ gst_osssink_change_state (GstElement *element)
     case GST_STATE_PAUSED_TO_READY:
       if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN))
         ioctl (osssink->common.fd, SNDCTL_DSP_RESET, 0);
-      /* FIXME: this is probably totally wrong ! we already inited once,
-       * and _init just resets variables ! wtay, if you agree then remove
-       * this code ;) */
-      /*
-      gst_osscommon_init (&osssink->common);
-      */
+      gst_osscommon_reset (&osssink->common);
       break;
     case GST_STATE_READY_TO_NULL:
       if (GST_FLAG_IS_SET (element, GST_OSSSINK_OPEN)) {
