@@ -33,6 +33,8 @@ static GstElementDetails gst_v4lelement_details = {
 /* V4lElement signals and args */
 enum {
   /* FILL ME */
+  SIGNAL_OPEN,
+  SIGNAL_CLOSE,
   LAST_SIGNAL
 };
 
@@ -63,6 +65,7 @@ enum {
   ARG_DISPLAY,
   ARG_VIDEOWINDOW,
   ARG_DO_OVERLAY,
+  ARG_SIGNAL,
 };
 
 
@@ -82,7 +85,7 @@ static gboolean              plugin_init                 (GModule            *mo
 
 
 static GstElementClass *parent_class = NULL;
-/*static guint gst_v4lelement_signals[LAST_SIGNAL] = { 0 }; */
+static guint gst_v4lelement_signals[LAST_SIGNAL] = { 0 };
 
 
 GType
@@ -140,6 +143,9 @@ gst_v4lelement_class_init (GstV4lElementClass *klass)
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_FREQUENCY,
     g_param_spec_ulong("frequency","frequency","frequency",
     0,G_MAXULONG,0,G_PARAM_READWRITE));
+  g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_SIGNAL,
+    g_param_spec_uint("signal","signal","signal",
+    0,65535,0,G_PARAM_READABLE));
 
   g_object_class_install_property(G_OBJECT_CLASS(klass), ARG_HAS_AUDIO,
     g_param_spec_boolean("has_audio","has_audio","has_audio",
@@ -205,6 +211,18 @@ gst_v4lelement_class_init (GstV4lElementClass *klass)
     g_param_spec_pointer("videowindow","videowindow","videowindow",
     G_PARAM_WRITABLE));
 
+  /* signals */
+  gst_v4lelement_signals[SIGNAL_OPEN] =
+    g_signal_new("open", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+                 G_STRUCT_OFFSET(GstV4lElementClass, open),
+                 NULL, NULL, g_cclosure_marshal_VOID__STRING,
+                 G_TYPE_NONE, 1, G_TYPE_STRING);
+  gst_v4lelement_signals[SIGNAL_CLOSE] =
+    g_signal_new("close", G_TYPE_FROM_CLASS(klass), G_SIGNAL_RUN_LAST,
+                 G_STRUCT_OFFSET(GstV4lElementClass, close),
+                 NULL, NULL, g_cclosure_marshal_VOID__STRING,
+                 G_TYPE_NONE, 1, G_TYPE_STRING);
+
   gobject_class->set_property = gst_v4lelement_set_property;
   gobject_class->get_property = gst_v4lelement_get_property;
 
@@ -275,11 +293,10 @@ gst_v4lelement_set_property (GObject      *object,
       break;
     case ARG_FREQUENCY:
       v4lelement->frequency = g_value_get_ulong(value);
-      if (GST_V4L_IS_OPEN(v4lelement) && !GST_V4L_IS_ACTIVE(v4lelement))
-      {
-        if (gst_v4l_has_tuner(v4lelement))
-          if (!gst_v4l_set_frequency(v4lelement, v4lelement->frequency))
-            return;
+      if (gst_v4l_has_tuner(v4lelement)){
+        if (!gst_v4l_set_frequency(v4lelement, v4lelement->frequency)){
+          return;
+	}
       }
       break;
     case ARG_MUTE:
@@ -295,7 +312,7 @@ gst_v4lelement_set_property (GObject      *object,
       v4lelement->mode = g_value_get_int(value);
       if (GST_V4L_IS_OPEN(v4lelement))
       {
-        if (!gst_v4l_set_audio(v4lelement, V4L_AUDIO_MODE, v4lelement->mute))
+        if (!gst_v4l_set_audio(v4lelement, V4L_AUDIO_MODE, v4lelement->mode))
           return;
       }
       break;
@@ -418,6 +435,12 @@ gst_v4lelement_get_property (GObject    *object,
           gst_v4l_get_frequency(v4lelement, &temp_ul);
       g_value_set_ulong(value, temp_ul);
       break;
+    case ARG_SIGNAL:
+      if (GST_V4L_IS_OPEN(v4lelement))
+        if (gst_v4l_has_tuner(v4lelement))
+          gst_v4l_get_signal(v4lelement, &temp_i);
+      g_value_set_uint(value, temp_i);
+      break;
     case ARG_HAS_AUDIO:
       g_value_set_boolean(value, FALSE);
       if (GST_V4L_IS_OPEN(v4lelement))
@@ -532,6 +555,10 @@ gst_v4lelement_change_state (GstElement *element)
       if (!gst_v4l_open(v4lelement))
         return GST_STATE_FAILURE;
 
+      g_signal_emit(G_OBJECT(v4lelement),
+                    gst_v4lelement_signals[SIGNAL_OPEN], 0,
+                    v4lelement->videodev);
+
       /* now, sync options */
       if (v4lelement->norm >= VIDEO_MODE_PAL &&
           v4lelement->norm < VIDEO_MODE_AUTO &&
@@ -581,6 +608,10 @@ gst_v4lelement_change_state (GstElement *element)
     case GST_STATE_READY_TO_NULL:
       if (!gst_v4l_close(v4lelement))
         return GST_STATE_FAILURE;
+
+      g_signal_emit(G_OBJECT(v4lelement),
+                    gst_v4lelement_signals[SIGNAL_CLOSE], 0,
+                    v4lelement->videodev);
       break;
   }
 
