@@ -54,7 +54,7 @@ static void                  gst_v4lmjpegsink_class_init   (GstV4lMjpegSinkClass
 static void                  gst_v4lmjpegsink_init         (GstV4lMjpegSink      *v4lmjpegsink);
 
 /* the chain of buffers */
-static GstPadConnectReturn   gst_v4lmjpegsink_sinkconnect  (GstPad               *pad,
+static GstPadLinkReturn   gst_v4lmjpegsink_sinkconnect  (GstPad               *pad,
                                                             GstCaps              *vscapslist);
 static void                  gst_v4lmjpegsink_chain        (GstPad               *pad,
                                                             GstBuffer            *buf);
@@ -166,7 +166,7 @@ gst_v4lmjpegsink_init (GstV4lMjpegSink *v4lmjpegsink)
   gst_element_add_pad (GST_ELEMENT (v4lmjpegsink), v4lmjpegsink->sinkpad);
 
   gst_pad_set_chain_function (v4lmjpegsink->sinkpad, gst_v4lmjpegsink_chain);
-  gst_pad_set_connect_function (v4lmjpegsink->sinkpad, gst_v4lmjpegsink_sinkconnect);
+  gst_pad_set_link_function (v4lmjpegsink->sinkpad, gst_v4lmjpegsink_sinkconnect);
 
   v4lmjpegsink->clock = NULL;
 
@@ -184,14 +184,14 @@ gst_v4lmjpegsink_init (GstV4lMjpegSink *v4lmjpegsink)
   v4lmjpegsink->bufferpool = gst_buffer_pool_new(
 				  NULL, 
 				  NULL,
-				  gst_v4lmjpegsink_buffer_new,
+				  (GstBufferPoolBufferNewFunction)gst_v4lmjpegsink_buffer_new,
 				  NULL,
 				  NULL,
 				  v4lmjpegsink);
 }
 
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_v4lmjpegsink_sinkconnect (GstPad  *pad,
                               GstCaps *vscapslist)
 {
@@ -202,13 +202,13 @@ gst_v4lmjpegsink_sinkconnect (GstPad  *pad,
 
   /* we are not going to act on variable caps */
   if (!GST_CAPS_IS_FIXED (vscapslist) || !GST_V4L_IS_OPEN(GST_V4LELEMENT(v4lmjpegsink)))
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
 
   /* in case the buffers are active (which means that we already
    * did capsnego before and didn't clean up), clean up anyways */
   if (GST_V4L_IS_ACTIVE(GST_V4LELEMENT(v4lmjpegsink)))
     if (!gst_v4lmjpegsink_playback_deinit(v4lmjpegsink))
-      return GST_PAD_CONNECT_REFUSED;
+      return GST_PAD_LINK_REFUSED;
 
   for (caps = vscapslist; caps != NULL; caps = vscapslist = vscapslist->next)
   {
@@ -231,11 +231,11 @@ gst_v4lmjpegsink_sinkconnect (GstPad  *pad,
     g_signal_emit (G_OBJECT (v4lmjpegsink), gst_v4lmjpegsink_signals[SIGNAL_HAVE_SIZE], 0,
       v4lmjpegsink->width, v4lmjpegsink->height);
 
-    return GST_PAD_CONNECT_OK;
+    return GST_PAD_LINK_OK;
   }
 
   /* if we got here - it's not good */
-  return GST_PAD_CONNECT_REFUSED;
+  return GST_PAD_LINK_REFUSED;
 }
 
 
@@ -263,6 +263,8 @@ gst_v4lmjpegsink_chain (GstPad    *pad,
   v4lmjpegsink = GST_V4LMJPEGSINK (gst_pad_get_parent (pad));
 
   if (v4lmjpegsink->clock) {
+    GstClockID id;
+
     GST_DEBUG (0,"videosink: clock wait: %llu", GST_BUFFER_TIMESTAMP(buf));
 
     jitter = 0; /* FIXME: jitter = gst_clock_current_diff(v4lmjpegsink->clock, GST_BUFFER_TIMESTAMP (buf)); */
@@ -270,7 +272,9 @@ gst_v4lmjpegsink_chain (GstPad    *pad,
     if (jitter > 500000 || jitter < -500000)
       GST_DEBUG (0, "jitter: %lld", jitter);
 
-    gst_element_clock_wait(GST_ELEMENT(v4lmjpegsink), v4lmjpegsink->clock, GST_BUFFER_TIMESTAMP(buf), NULL);
+    id = gst_clock_new_single_shot_id (v4lmjpegsink->clock, GST_BUFFER_TIMESTAMP(buf));
+    gst_element_clock_wait(GST_ELEMENT(v4lmjpegsink), id, NULL);
+    gst_clock_id_free (id);
   }
 
   if (GST_BUFFER_POOL(buf) == v4lmjpegsink->bufferpool)
