@@ -1,11 +1,6 @@
-
 #ifndef __GST_XIMAGE_H__
 #define __GST_XIMAGE_H__
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
 #include "config.h"
 
 #define USE_SHM
@@ -21,6 +16,8 @@ extern "C" {
 
 #include "gstimage.h"
 
+G_BEGIN_DECLS
+
 #define GST_XIMAGE(img)		((GstXImage*)(img))
 
 typedef struct _GstXImage	      GstXImage;
@@ -50,9 +47,6 @@ struct _GstXImage
 
 GstXImage*	_gst_ximage_new    	(GstXWindow *window, gint width, gint height);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
-
+G_END_DECLS
 
 #endif /* __GST_XIMAGE_H__ */
