@@ -1,14 +1,11 @@
- 
-#ifndef __GDK_XVIMAGE_H__
-#define __GDK_XVIMAGE_H__
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+#ifndef __GST_XWINDOW_H__
+#define __GST_XWINDOW_H__
 
 #include <glib.h>
 #include <X11/Xlib.h>
 
+G_BEGIN_DECLS
+
 typedef struct _GstXWindow	      GstXWindow;
 
 #define GST_XWINDOW_XID(window)	((window)->win)
@@ -31,8 +28,6 @@ void 		_gst_xwindow_destroy 	(GstXWindow *window);
 
 void 		_gst_xwindow_resize 	(GstXWindow *window, gint width, gint height);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
-#endif /* __GDK_XVIMAGE_H__ */
+#endif /* __GST_XWINDOW_H__ */
